#include <nds.h>
#include <stdio.h>

static int old_x = 0;
static int old_y = 0;
static int old_width = 0;
static int old_height = 0;

static int shape_x = 0;
static int shape_y = 0;
static int shape_width = 20;
static int shape_height = 20;

void draw(int x, int y, uint16* buf, uint16 c) {
    buf += y * SCREEN_WIDTH + x;
    for(int i = 0; i < shape_height; i++) {
        uint16* line = buf + SCREEN_WIDTH * i;
        for(int j = 0; j < shape_width; j++) {
            *line++ = c;
        }
    }
}

void clean(int x, int y, uint16* buf, uint16 c) {
    buf += y * SCREEN_WIDTH + x;
    for(int i = 0; i < old_height; i++) {
        uint16* line = buf + SCREEN_WIDTH * i;
        for(int j = 0; j < old_width; j++) {
            *line++ = c;
        }
    }
}

int main(void) {
    //turn on the power to the system
    powerON(POWER_ALL);
    irqInit();
    irqEnable(IRQ_VBLANK);
    
    //set main display to render directly from the frame buffer
    videoSetMode(MODE_FB0);
    
    // set the sub background up for text display (we could just print to one
    // of the main display text backgrounds just as easily
    videoSetModeSub(MODE_0_2D | DISPLAY_BG0_ACTIVE); //sub bg 0 will be used to print text
    
    // memory so another vram bank must be used and mapped consecutivly
    vramSetMainBanks( VRAM_A_LCD, VRAM_B_LCD,
                        VRAM_C_SUB_BG , VRAM_D_LCD);
    
    // set up text background for text
    SUB_BG0_CR = BG_MAP_BASE(31);

    BG_PALETTE_SUB[255] = RGB15(31,31,31);//by default font will be rendered with color 255

    //consoleInit() is a lot more flexible but this gets you up and running quick
    consoleInitDefault((u16*)SCREEN_BASE_BLOCK_SUB(31), (u16*)CHAR_BASE_BLOCK_SUB(0), 16);
    
    while(1) {
        iprintf("\x1b[10;0Hx,y = %4d, %4d\n", shape_x, shape_y);
        iprintf("width,ghight = %4d, %4d\n", shape_width, shape_height);
        iprintf("menseki = %4d\n", shape_width*shape_height);
        
        old_x = shape_x;
        old_y = shape_y;
		old_width = shape_width;
		old_height = shape_height;
        
		scanKeys();
		u32 keys = keysHeld();

		if ( keys & KEY_L ){ shape_height = 20; }
		if ( keys & KEY_R ){ shape_width = 20; }

		if ( keys & KEY_LEFT ){ shape_x--; }
		if ( keys & KEY_RIGHT ){ shape_x++; }
		if ( keys & KEY_UP ){ shape_y--; }
		if ( keys & KEY_DOWN ){ shape_y++; }

		if ( keys & KEY_A ){ shape_width++; }
		if ( keys & KEY_B ){ shape_height++; }

		if( keys & KEY_X ){ shape_height--; }
		if( keys & KEY_Y ){ shape_width--; }
		
		
		
        if(shape_x + shape_width >= SCREEN_WIDTH) {
            shape_x = SCREEN_WIDTH - shape_width;
            if(shape_x <= 0){ shape_x = 0; shape_width = SCREEN_WIDTH; }
        }else if(shape_x <= 0){
        	shape_x = 0;
        }
        if(shape_y + shape_height >= SCREEN_HEIGHT) {
            shape_y = SCREEN_HEIGHT - shape_height;
            if(shape_y <= 0){ shape_y = 0; shape_height = SCREEN_HEIGHT; }
        }else if(shape_y <= 0){
        	shape_y = 0;
        }
        if(shape_width <= 0) shape_width = 1;
        if(shape_height <= 0) shape_height = 1;
        
        clean(old_x, old_y,VRAM_A, RGB15(0,0,0));
        draw(shape_x, shape_y, VRAM_A, RGB15(0, 0, 15));
        swiWaitForVBlank();
    }
    
    return 0;
}
